/*  Project fancy_ed
    Terry Richards Software
    Copyright  1994. All Rights Reserved.

    SUBSYSTEM:    fancy_ed.exe Application
    FILE:         fncy_dwn.cpp
    AUTHOR:       Terry Richards


    OVERVIEW
    ========
    Source file for implementation of fancy_edWindow (TWindow).      
*/


#include <owl\owlpch.h>
#pragma hdrstop

#include "fncy_dap.h"
#include "fncy_dwn.h"
#include "fncy_dwn.h"

#include <stdio.h>


//{{fancy_edWindow Implementation}}


//
// Build a response table for all messages/commands handled
// by fancy_edWindow derived from TEditClient.
//
DEFINE_RESPONSE_TABLE1(fancy_edWindow, TEditClient)
//{{fancy_edWindowRSP_TBL_BEGIN}}
    EV_WM_GETMINMAXINFO,
    EV_COMMAND (CM_CHOOSEFONT,CMChooseFont),
    EV_COMMAND (CM_SELECTALL,CM_SELECTALLMethod),
    EV_COMMAND (CM_REVERT,CM_REVERTMethod),
    EV_COMMAND_ENABLE (CM_REVERT,CM_REVERTEnable),
    EV_WM_KEYDOWN,
    EV_WM_SETFOCUS,
    EV_WM_RBUTTONDOWN,
//{{fancy_edWindowRSP_TBL_END}}
END_RESPONSE_TABLE;


//////////////////////////////////////////////////////////
// fancy_edWindow
// ==========
// Construction/Destruction handling.
fancy_edWindow::fancy_edWindow (TWindow* parent, const char far* title, TModule* /*module*/)
    : TEditClient(TYPESAFE_DOWNCAST(parent,TMDIChild), title)    //TR - changed parent class
{
    // INSERT>> Your constructor code here.


    MyFont = new TFont("Arial",-14);

}


fancy_edWindow::~fancy_edWindow ()
{
    Destroy();

    // INSERT>> Your destructor code here.

    delete MyFont;

}


//
// Paint routine for Window, Printer, and PrintPreview for an TWindow client.
//
void fancy_edWindow::Paint (TDC& DC, bool fErase, TRect& rect)
{
    DC.SelectObject(*MyFont);
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);
    if (theApp) {
        // Only paint if we're printing and we have something to paint, otherwise do nothing.
        if (theApp->Printing && theApp->Printer && !rect.IsEmpty()) {
            // Use pageSize to get the size of the window to render into.  For a Window it's the client area,
            // for a printer it's the printer DC dimensions and for print preview it's the layout window.
            //TR - I don't think this would be right for a printer that wants banding
            //     we don't use it anyway.
            //TSize   pageSize(rect.right - rect.left, rect.bottom - rect.top);

            TPrintDialog::TData &printerData = theApp->Printer->GetSetup();

            // Compute the number of pages to print.
            //TR - removed next 2 lines
            //printerData.MinPage = 1;
            //printerData.MaxPage = 1;

            // INSERT>> Special printing code goes here.

            TEXTMETRIC  tm;
            int fHeight = (DC.GetTextMetrics(tm) == true) ? tm.tmHeight + tm.tmExternalLeading : 10;
            int lastLine = printerData.FromPage /* page # */ * printerData.ToPage /* lines per page */;
            int firstLine = lastLine - printerData.ToPage;
            TPoint p(0,0);

            if (lastLine > pLines.GetItemsInContainer())
                lastLine = pLines.GetItemsInContainer();

            for (int i = firstLine;i < lastLine;i++){

				p.y = (i-firstLine)*fHeight;
				DC.TabbedTextOut(p,pLines[i]->c_str(),-1,NumTabs,TabArray,0);

            }

        } else {
            // INSERT>> Normal painting code goes here.
            //TR - we can just pass through to the editor
            TEditClient::Paint (DC,fErase,rect);
        }
    }
    DC.RestoreFont();
}


void fancy_edWindow::EvGetMinMaxInfo (MINMAXINFO far& minmaxinfo)
{
    fancy_edApp *theApp = TYPESAFE_DOWNCAST(GetApplication(), fancy_edApp);
    if (theApp) {
        if (theApp->Printing) {
            minmaxinfo.ptMaxSize = TPoint(0, pLines.GetItemsInContainer());
            minmaxinfo.ptMaxTrackSize = TPoint(0, 0);
            return;
        }
    }
    TWindow::EvGetMinMaxInfo(minmaxinfo);
}

HFONT fancy_edWindow::GetWindowFont(){

    return (HFONT)(HGDIOBJ)*MyFont;

}

void fancy_edWindow :: CMChooseFont(){

TClientDC                hdc1(HWindow);
TChooseFontDialog::TData FontData;
char                     szFontStyle[LF_FACESIZE]="";
int                      iPoints;
LOGFONT                  lf;
TEXTMETRIC               tm;

    MyFont->GetObject(lf); //Fill in logfont data from existing font

    memset(&FontData,0,sizeof(FontData));
    memcpy(&FontData.LogFont,&lf,sizeof(LOGFONT));
    strcpy(szFontStyle,"");
    FontData.Style = szFontStyle;
    iPoints        = -14;
    FontData.Color   = 0x000000L;  // color of text sample will be black
    FontData.SizeMin    = 0;
    FontData.SizeMax    = 0x7fff;
    FontData.Flags = CF_USESTYLE | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
    FontData.LogFont.lfHeight = MulDiv(hdc1.GetDeviceCaps(LOGPIXELSY),iPoints,72);

    if (TChooseFontDialog(this,FontData).Execute() != IDOK)
        return;

    // FontData.LogFont has the selected font
    delete MyFont;
    MyFont = new TFont(&FontData.LogFont);
    SetFont(hdc1);
    hdc1.GetTextMetrics(tm);
    LineHeight = tm.tmHeight + tm.tmExternalLeading;
    CharAveWidth = tm.tmAveCharWidth;
    Scroller->SetUnits(CharAveWidth,LineHeight);
    //Reset the caret size
    EvKillFocus(HWindow);
    EvSetFocus(HWindow);

    Invalidate(TRUE);

}

void fancy_edWindow :: SetFont(TDC& DC){

    DC.SelectObject(*MyFont);

}

void fancy_edWindow :: SizeChanged(BOOL /*JustFocus*/){

char    Msg[10];
ulong   Count=0;
int     i;

    //Count characters. Add 2 for each new line (\r\n)

    for (i=0;i<pLines.GetItemsInContainer();i++)
        Count += pLines[i]->length() + 2;

    ultoa(Count,Msg,10);

    fancy_edApp* App = TYPESAFE_DOWNCAST(GetApplication(),fancy_edApp);
    TTextGadget* Widget = TYPESAFE_DOWNCAST((*(App->sb))[2],TTextGadget); //This looks like line noise!

    Widget->SetText(Msg);

}

void fancy_edWindow :: PosChanged(BOOL /*JustFocus*/){

char    Msg[10];
char    Msg2[10];

    //Build up Line:Col message

    itoa(CaretPos.GetLine()+1,Msg,10);
    itoa(CaretPos.GetCol()+1,Msg2,10);

    strcat(Msg,":");
    strcat(Msg,Msg2);

    fancy_edApp* App = TYPESAFE_DOWNCAST(GetApplication(),fancy_edApp);
    TTextGadget* Widget = TYPESAFE_DOWNCAST((*(App->sb))[1],TTextGadget); //This looks like line noise!

    Widget->SetText(Msg);

}

void fancy_edWindow :: CM_SELECTALLMethod(){

    //Select everything

    HiStart.Home(TRUE);
    HiEnd.End(TRUE);
    CaretPos = HiStart;

    Invalidate();
    PosChanged();

}

void fancy_edWindow::EvKeyDown (uint key, uint repeatCount, uint flags)
{
    TEditClient::EvKeyDown(key, repeatCount, flags);

    if (key == VK_INSERT){

        fancy_edApp* App = TYPESAFE_DOWNCAST(GetApplication(),fancy_edApp);
        App->sb->SetModeIndicator(TStatusBar::Overtype,OverType);

    }

}


void fancy_edWindow::EvSetFocus (HWND hWndLostFocus )
{
    TEditClient::EvSetFocus(hWndLostFocus );

    fancy_edApp* App = TYPESAFE_DOWNCAST(GetApplication(),fancy_edApp);
    App->sb->SetModeIndicator(TStatusBar::Overtype,OverType);

}

void fancy_edWindow :: CM_REVERTMethod(){

    pLines.Flush();
    ClearUndoStack();
    LoadFile();
    IsDirty = FALSE;
    CaretPos.Home(TRUE);
    HiStart = CaretPos;
    HiEnd   = CaretPos;
    ScrollToCaret();
    SizeChanged();
    Invalidate();

}

void fancy_edWindow :: CM_REVERTEnable(TCommandEnabler& commandHandler){

    commandHandler.Enable(File[0] && IsDirty);

}

void fancy_edWindow :: EvRButtonDown(UINT /*modKeys*/, TPoint& point){

  TPoint lp = point;
  TPopupMenu *menu = new TPopupMenu();

  menu->AppendMenu(MF_STRING, CM_FILESAVE, "Save");
  menu->AppendMenu(MF_STRING, CM_FILESAVEAS, "Save As...");
  menu->AppendMenu(MF_SEPARATOR, 0, 0);
  menu->AppendMenu(MF_STRING, CM_EDITUNDO, "Undo");
  menu->AppendMenu(MF_STRING, CM_EDITCUT, "Cut");
  menu->AppendMenu(MF_STRING, CM_EDITCOPY, "Copy");
  menu->AppendMenu(MF_STRING, CM_EDITPASTE, "Paste");
  menu->AppendMenu(MF_STRING, CM_EDITDELETE, "Delete");
  menu->AppendMenu(MF_STRING, CM_EDITCLEAR, "Clear All");
  menu->AppendMenu(MF_SEPARATOR, 0, 0);
  menu->AppendMenu(MF_STRING, CM_EDITFIND, "Find...");
  menu->AppendMenu(MF_STRING, CM_EDITREPLACE, "Replace...");
  menu->AppendMenu(MF_STRING, CM_EDITFINDNEXT, "Next");

  ClientToScreen(lp);
  menu->TrackPopupMenu(TPM_LEFTALIGN|TPM_RIGHTBUTTON, lp, 0, *this);

  delete menu;

}

